//Joo Won Park
//SuperCollider HID mappings
//Refer to
//www.joowonpark.net/hidcontrollers

//PS5 DualSense Bluetooth
//For Mac OSX
//Last Tested on Feb 2025

//Evaluate the codes inside ( )
(
//1. Controller Initializations
~id=[1356,3302];
~controller=[
	"Square","X","Circle","Trianle","L1","R1","L2","R2","Share","Option","L3","R3","Logo","Trackpad",
	"L3 x-axis","L3 y-axis","R3 x-axis","R3 y-axis",
	"Direction",
	"L2 continuous","R2 continuous"
];

~directionpad=["up (0)","up right (1)","right (2)","down right (3)","down (4)","down left (5)","left (6)","up left (7)","release (8)"];

~directionelement=18;

HID.findAvailable; //must
HID.postAvailable;

//if the controller disconnects, evaluate the following line again after reconnecting the device
~myhid = HID.open( ~id[0], ~id[1] ); //vendor id, product id
//~myhid = HID.close( ~id[0], ~id[1] );

//functions
~whichsurface={
	arg element;
	if (element==~directionelement,
		{
			HIDFunc.element({
				|...args|
				element.do{
					arg count;
					if (args[1]==count,{[element,"Direction Pad",~directionpad[count]].postln;});
				}
			}, [element]);
		},
		{
			HIDFunc.element({
				|...args|
				[element,~controller.[element],args[1]].postln;
			}, [element]);
	});

};


~allcontrol={
	["Element Number","Surface Name","Value"].postln;
	~controller.size.do{
		arg count;	~whichsurface.(count);
	}
};

);


(
//2. Test the Controller
//Refert to
//https://docs.google.com/spreadsheets/d/1dPizIDVm62KGQAMp19nvF77h2tVvNxJ8yXq1ndL0bRM/edit?usp=sharing
//press command+. to stop receiving messages

//Get all inputs. element number, control surface name, and its value will appear on the Post window
~allcontrol.();

//Get individual inputs
["Element Number","Surface Name","Value"].postln;
//buttons
~whichsurface.(0);
~whichsurface.(1);
~whichsurface.(2);
~whichsurface.(3);
~whichsurface.(4);
~whichsurface.(5);
~whichsurface.(6);
~whichsurface.(7);
~whichsurface.(8);
~whichsurface.(9);
~whichsurface.(10);
~whichsurface.(11);
~whichsurface.(12);
~whichsurface.(13);

//directional pad
~whichsurface.(18);

//continuous controllers
~whichsurface.(14);
~whichsurface.(15);
~whichsurface.(16);
~whichsurface.(17);
~whichsurface.(19);
~whichsurface.(20);
);


//3. Audio Example
s.waitForBoot{
	SynthDef("TestNoise",{
		arg vol,pan;
		var sound;
		sound=WhiteNoise.ar(vol*0.3);
		pan=Pan2.ar(sound,pan);
		Out.ar(0,pan);
	}).add;
};

//evaluate the follwoing codes
(
~testsound=Synth("TestNoise",[\vol,0.2]);

//add parameter changes inside HIDFunc.element
HIDFunc.element({
	arg floatnum,intnum;
	~testsound.set(\vol,intnum*0.5+0.2)
}, [0]); //type element number here (default: square)

HIDFunc.element({
	arg floatnum,intnum;
	var scale;
	scale=intnum.linlin(0,255,-1.0,1.0); //some values need scalining
	~testsound.set(\pan,scale)
}, [16]); // type element number here (default: right joystick x-axis)

//command+. to stop
);

