//Joo Won Park
//SuperCollider HID mappings
//Refer to
//www.joowonpark.net/hidcontrollers

//Logitech FS310 USB
//Set FA310 to the D Mode
//For Mac OSX
//Last Tested on Feb 2025

//Evaluate the codes inside ( )
(
//1. Controller Initializations
~id=[1133,49686];
~controller=[
	"X","A","B","Y","LB","RB","LT","RT","Back","Start","LSB","RSB","NA",
	"LSB x-axis","LSB y-axis","RSB x-axis","RSB y-axis",
	"Direction"
];

~directionpad=["up (0)","up right (1)","right (2)","down right (3)","down (4)","down left (5)","left (6)","up left (7)","release (8)"];

~directionelement=17;

//
HID.findAvailable; //must
HID.postAvailable;

//if the controller disconnects, evaluate the following line again after reconnecting the device
~myhid = HID.open( ~id[0], ~id[1] ); //vendor id, product id
//Post Window shows an error message if a controller with the same id is already connected.
//If that is the case, disregard the error message and test the controller

//functions
~whichsurface={
	arg element;
	if (element==~directionelement,
		{
			HIDFunc.element({
				|...args|
				element.do{
					arg count;
					if (args[1]==count,{[element,"Direction Pad",~directionpad[count]].postln;});
				}
			}, [element]);
		},
		{
			HIDFunc.element({
				|...args|
				[element,~controller.[element],args[1]].postln;
			}, [element]);
	});

};


~allcontrol={
	["Element Number","Surface Name","Value"].postln;
	~controller.size.do{
		arg count;	~whichsurface.(count);
	}
};

);


(
//2. Test the controller
//Refert to
//https://docs.google.com/spreadsheets/d/1dPizIDVm62KGQAMp19nvF77h2tVvNxJ8yXq1ndL0bRM/edit?usp=sharing
//press command+. to stop receiving messages

//Get all inputs. element number, control surface name, and its value will appear on the Post window
~allcontrol.();

//Get individual inputs
["Element Number","Surface Name","Value"].postln;
//buttons
~whichsurface.(0);
~whichsurface.(1);
~whichsurface.(2);
~whichsurface.(3);
~whichsurface.(4);
~whichsurface.(5);
~whichsurface.(6);
~whichsurface.(7);
~whichsurface.(8);
~whichsurface.(9);
~whichsurface.(10);
~whichsurface.(11);

//directional pad
~whichsurface.(17);

//continuous controllers
~whichsurface.(13);
~whichsurface.(14);
~whichsurface.(15);
~whichsurface.(16);

);


//3. Audio example
s.waitForBoot{
	SynthDef("TestNoise",{
		arg vol,pan;
		var sound;
		sound=WhiteNoise.ar(vol*0.3);
		pan=Pan2.ar(sound,pan);
		Out.ar(0,pan);
	}).add;
};

//evaluate the follwoing codes
(
~testsound=Synth("TestNoise",[\vol,0.2]);

//add parameter changes inside HIDFunc.element
HIDFunc.element({
	arg floatnum,intnum;
	~testsound.set(\vol,intnum*0.5+0.2)
}, [3]); //type element number here (default: Y)

HIDFunc.element({
	arg floatnum,intnum;
	var scale;
	scale=intnum.linlin(0,255,-1.0,1.0); //some values need scalining
	~testsound.set(\pan,scale)
}, [15]); // type element number here (default: right joystick x-axis)

//command+. to stop
);

