//Joo Won Park
//SuperCollider HID mappings
//Refer to
//www.joowonpark.net/hidcontrollers

//XBox Bluetooth
//For Mac OSX
//Last Tested on Feb 2025

//Evaluate the codes inside ( )
(
//1. Controller Initializations
~id=[1118,765];
~controller=[
	"LSB x-axis","LSB y-axis","RSB x-axis","RSB y-axis",
4,5,6,7,8,9,	"A","B",12,"X","Y",15,"LB","RB",18,19,20,"Start",22,23,24,"Back","LT Continuous","RT Continuous","Direction"];

~directionpad=["release (0)","up (1)","up right (2)","right (3)","down right (4)","down (5)","down left (6)","left (7)","up left (8)",];

~directionelement=28;

//
HID.findAvailable; //must
HID.postAvailable;

//if the controller disconnects, evaluate the following line again after reconnecting the device
~myhid = HID.open( ~id[0], ~id[1] ); //vendor id, product id
//Post Window shows an error message if a controller with the same id is already connected.
//If that is the case, disregard the error message and test the controller

//functions
~whichsurface={
	arg element;
	if (element==~directionelement,
		{
			HIDFunc.element({
				|...args|
				element.do{
					arg count;
					if (args[1]==count,{[element,"Direction Pad",~directionpad[count]].postln;});
				}
			}, [element]);
		},
		{
			HIDFunc.element({
				|...args|
				[element,~controller.[element],args[1]].postln;
			}, [element]);
	});

};


~allcontrol={
	["Element Number","Surface Name","Value"].postln;
	~controller.size.do{
		arg count;	~whichsurface.(count);
	}
};

);


(
//2. Test the Controller
//Refert to
//https://docs.google.com/spreadsheets/d/1dPizIDVm62KGQAMp19nvF77h2tVvNxJ8yXq1ndL0bRM/edit?usp=sharing
//press command+. to stop receiving messages

//Get all inputs. element number, control surface name, and its value will appear on the Post window
~allcontrol.();

//Get individual inputs
["Element Number","Surface Name","Value"].postln;
//buttons
~whichsurface.(10);
~whichsurface.(11);
~whichsurface.(13);
~whichsurface.(14);
~whichsurface.(16);
~whichsurface.(17);
~whichsurface.(21);
~whichsurface.(25);


//directional pad
~whichsurface.(28);

//continuous controllers
~whichsurface.(0);
~whichsurface.(1);
~whichsurface.(2);
~whichsurface.(3);
~whichsurface.(26);
~whichsurface.(27);

);


//3. Audio Example
s.waitForBoot{
	SynthDef("TestNoise",{
		arg vol,pan;
		var sound;
		sound=WhiteNoise.ar(vol*0.3);
		pan=Pan2.ar(sound,pan);
		Out.ar(0,pan);
	}).add;
};

//evaluate the follwoing codes
(
~testsound=Synth("TestNoise",[\vol,0.2]);

//add parameter changes inside HIDFunc.element
HIDFunc.element({
	arg floatnum,intnum;
	~testsound.set(\vol,intnum*0.5+0.2)
}, [14]); //type element number here (default: Y)

HIDFunc.element({
	arg floatnum,intnum;
	var scale;
	scale=intnum.linlin(0,65535,-1.0,1.0); //some values need scalining
	~testsound.set(\pan,scale)
}, [2]); // type element number here (default: right joystick x-axis)

//command+. to stop
);

